import pandas as pd

# def skeleton(data,Log):
#     data["new_column"] = (data["a"] * data["b"] / data["c"] - data["e"] + data["d"].sum())
#     Log += "\n## Did something\n A*B/(C - E) + sum(D), cool\n\n"
#     columns = ["new_column"]
#     return data, Log, columns

def DegreeDays(data,Log):
    """! Calculate Degreedays based on outside temperature and livingroom temperature

    @param data  Standardized input (pandas)dataframe. It's content can be derived from <input_data>.metadata
    @param Log  This Log will be compiled to .html, you can add results to this.

    @return data     A new dataframe with the applied transformations
    @return Log      The log with new entries
    @return columns  New columns added to the dataframe
    """
    data["DegreeDays"] = (data["Tlv"] - data["Tamb"]).resample("1d").mean().interpolate()
    Log += "\n## Added livingroom degree-days\n Tlv - Tamb \n---------------------------\n"
    columns = ["DegreeDays"]
    return data, Log, columns



import markdown
import subprocess
import os
import sys
from TracknTrace.preprocessor import preProcessor


## Log = ""
def main():

    data, Log, instance = preProcessor.main()
    ## print(Log)
    with open('{}_result.md'.format(instance), 'w') as f:
        f.write(Log)
    html_string = markdown.markdown(Log, extensions=['tables'])
    ## print(html_string)
    with open('{}_result.html'.format(instance), 'w') as f:
        f.write(html_string)

    subprocess.Popen('{start} {path}'.format(  ## Open generated report in firefox
        start='firefox', path='{}_result.html'.format(instance)), shell=True)

if __name__ == "__main__":
    print(("%s is being run directly" % __name__))
    try:
        main()
    except RuntimeError:
        print("Error")
        os._exit(os.EX_OK)
else:
    print("---\n{}\n---\nV0.0.1 is being imported\n---".format(sys.argv[0]))



from setuptools import setup
from setuptools.command.install import install

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(name='TracknTrace',
      version='0.1.0',
      description='Track building performance and Trace errors',
      long_description=long_description,
      long_description_content_type="text/markdown",
      include_package_data=True,
      classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Topic :: Scientific/Engineering :: Atmospheric Science',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Education'
      ],
      keywords='Track Trace building performance errors gap analysis DSMR KNMI blame',
      url='https://github.com/Kwabratseur/TracknTrace',
      author='Jeroen van \'t Ende',
      author_email='jeroen.vantende@outlook.com',
      license='MIT', # mit should be fine, looking at the deps.. only those BSD-3
      packages=['TracknTrace',
                'TracknTrace.preprocessor',
                'TracknTrace.categorizer'],
      package_data={"TracknTrace": ["*.metadata", "*.txt", "*.xlsx"]},
      install_requires=[
        'pandas', # BSD-3
        'numpy', # BSD-3
        'markdown',
        'matplotlib',
        'pillow',
        'pytexit',
        'seaborn',
        'scikit-learn',
        'mufit',
        'openpyxl',
        'tabulate',
        'latex'
      ],
      entry_points = {
        'console_scripts': ['TracknTrace=TracknTrace.wrapper:main'],#,
                            #'preprocessor=TracknTrace.preprocessor',
                            #'categorizer=TracknTrace.categorizer'],
      },
      zip_safe=False)



import pandas as pd
import argparse
import configparser

pd.options.plotting.backend = "plotly"
pd.options.mode.chained_assignment = None

def EventCategorizerFcn(categories, data, cats, target): # list, data, df
    """Creates map of category:data and sums results per category

    @param categories  data column: category translation pairs
    @param data  original data to categorize
    @param cats  Scaling of detected event
    @param target  The target trends in data to dp event categorization on.

    @return df  dataframe which dumps original data and replaces with event detections for target"""
    df = pd.DataFrame(columns = categories[0][1:], index = data.index)
    df =  df.fillna(0.0)
    print(df)
    TargetOrigin = target.split("_")[1]
    for i in data[[target]].T:
        #print(i, data[[target]].T[i], float(data[[target]].T[i])*cats.loc[TargetOrigin])
        df.T[i] += float(data[[target]].T[i].iloc[0])*cats.loc[TargetOrigin]# ## $ FINALLY TYHIS DOESD OETGIOGFEIHAOFAWIO{WFG
    return df

def DoAllCategories(categories, data, cats, ScanList = None): # list, data, df
    """Creates map of category:data and sums results per category for all data

    @param categories  data column: category translation pairs
    @param data  original data to categorize
    @param cats  Scaling of detected event

    @return df  dataframe which dumps original data and replaces with event detections for all data"""
    df = pd.DataFrame(columns = categories[0], index = data.index)
    df =  df.fillna(0.0)
    print(df)
    progress = len(data.columns)
    for z,j in enumerate(data.columns):
        print("doing {}, {}%".format(j,(z/progress)*100))
        try:
            if j != "normevents":
                TargetOrigin = j.split("_")[1]
                if ScanList is None or TargetOrigin in ScanList:
                    for i in data[[j]].T:

                        #df.T[i] += abs(float(data[[j]].T[i].iloc[0]))*cats.loc[TargetOrigin]# ## $ FINALLY TYHIS DOESD OETGIOGFEIHAOFAWIO{WFG
                        #print("Transponate:",df.T[i])
                        #print("loc:",df.loc[i,])
                        #print(df.T[i])
                        df.loc[i] = df.loc[i,] + abs(float(data[[j]].loc[i,].iloc[0]))*cats.loc[TargetOrigin]
                        #print(i, data[[j]].loc[i,], float(data[[j]].loc[i,].iloc[0])*cats.loc[TargetOrigin], df.loc[i])
        except KeyError:
            print("{} does not exist".format(j))
    return df

parser = argparse.ArgumentParser()
parser.add_argument("instance", type=str,
                    help="Input Instance")
args = parser.parse_args()


EventData = pd.read_csv("{}_Events_Export.csv".format(args.instance))
EventData.set_index("DateTime", inplace=True)

Finalcategories = pd.read_csv("{}_categories_Export.csv".format(args.instance))
Finalcategories.set_index("Unnamed: 0", inplace=True)


print(EventData.describe())
print(EventData)

print(Finalcategories.describe())
print(Finalcategories)

print([list(Finalcategories),Finalcategories.values.tolist()])

print("lets try to do the work!")

print(Finalcategories.index)

config = configparser.ConfigParser()
config.optionxform = lambda option: option
result_list = config.read("{}.metadata".format(args.instance))
Scanlist = config["eventdetection"]["Scanlist"].split(",")
print("Items included in performance gap scan: {}".format(Scanlist))

#df = EventCategorizerFcn([list(Finalcategories),Finalcategories.values.tolist()], EventData, Finalcategories, "event_Sbd_sw_24_1.2")
df = DoAllCategories([list(Finalcategories),Finalcategories.values.tolist()], EventData, Finalcategories, ScanList = Scanlist)

print(df)

fig = df.plot()
fig.write_html("{}_FinalCategories.html".format(args.instance), auto_open=True)
dfnorm = df/df.max()

fig = dfnorm.plot()
fig.write_html("{}_FinalCategories_normalized.html".format(args.instance), auto_open=True)
